@@ -1227,11 +1227,14 @@ class TreePath(Gtk.TreePath):
         return other is None or self.compare(other) >= 0
 
     def __iter__(self):
-        return (int(part) for part in str(self).split(':'))
+        return iter(self.get_indices())
 
     def __len__(self):
         return self.get_depth()
 
+    def __getitem__(self, index):
+        return self.get_indices()[index]
+
 TreePath = override(TreePath)
 __all__.append('TreePath')
 
@@ -1396,6 +1399,13 @@ class TreeViewColumn(Gtk.TreeViewColumn):
     def set_cell_data_func(self, cell_renderer, func, func_data=None):
         super(TreeViewColumn, self).set_cell_data_func(cell_renderer, func, func_data)
 
+    def set_attributes(self, cell_renderer, **attributes):
+        Gtk.CellLayout.clear_attributes(self, cell_renderer)
+
+        for (name, value) in attributes.items():
+            Gtk.CellLayout.add_attribute(self, cell_renderer, name, value)
+
+
 TreeViewColumn = override(TreeViewColumn)
 __all__.append('TreeViewColumn')
 
