@@ -81,9 +81,13 @@ _pygi_closure_assign_pyobj_to_out_argument (gpointer out_arg, PyObject *object,
            interface_type = g_base_info_get_type (interface);
 
            if (!g_type_info_is_pointer (type_info) &&
-	       interface_type == GI_INFO_TYPE_STRUCT) {
-               gsize item_size = _pygi_g_type_info_size (type_info);
-               memcpy (out_arg, arg.v_pointer, item_size);
+               interface_type == GI_INFO_TYPE_STRUCT) {
+               if (object == Py_None) {
+                   arg.v_pointer = NULL;
+               } else {
+                   gsize item_size = _pygi_g_type_info_size (type_info);
+                   memcpy (out_arg, arg.v_pointer, item_size);
+               }
                break;
            }
         }
@@ -408,9 +412,6 @@ _pygi_closure_handle (ffi_cif *cif,
 {
     PyGILState_STATE state;
     PyGICClosure *closure = data;
-    GITypeTag  return_tag;
-    GITransfer return_transfer;
-    GITypeInfo *return_type;
     PyObject *retval;
     PyObject *py_args;
     GIArgument *out_args = NULL;
@@ -419,10 +420,6 @@ _pygi_closure_handle (ffi_cif *cif,
       may be executing python code */
     state = PyGILState_Ensure();
 
-    return_type = g_callable_info_get_return_type (closure->info);
-    return_tag = g_type_info_get_tag (return_type);
-    return_transfer = g_callable_info_get_caller_owns (closure->info);
-
     if (!_pygi_closure_convert_arguments ( (GICallableInfo *) closure->info, args,
                                            closure->user_data,
                                            &py_args, &out_args)) {
@@ -443,7 +440,6 @@ _pygi_closure_handle (ffi_cif *cif,
 
 end:
     g_free (out_args);
-    g_base_info_unref ( (GIBaseInfo*) return_type);
 
     PyGILState_Release (state);
 
