@@ -369,7 +369,7 @@ _pygi_marshal_from_py_int64 (PyGIInvokeState   *state,
                              GIArgument        *arg)
 {
     PyObject *py_long;
-    long long long_;
+    gint64 long_;
 
     if (!PyNumber_Check (py_arg)) {
         PyErr_Format (PyExc_TypeError, "Must be number, not %s",
@@ -383,10 +383,10 @@ _pygi_marshal_from_py_int64 (PyGIInvokeState   *state,
 
 #if PY_VERSION_HEX < 0x03000000
     if (PyInt_Check (py_long))
-        long_ = PyInt_AS_LONG (py_long);
+        long_ = (gint64) PyInt_AS_LONG (py_long);
     else
 #endif
-        long_ = PyLong_AsLongLong (py_long);
+        long_ = (gint64) PyLong_AsLongLong (py_long);
 
     Py_DECREF (py_long);
 
@@ -418,7 +418,7 @@ _pygi_marshal_from_py_int64 (PyGIInvokeState   *state,
             Py_DECREF (py_str);
         }
 
-        PyErr_Format (PyExc_ValueError, "%s not in range %ld to %ld",
+        PyErr_Format (PyExc_ValueError, "%s not in range %" G_GINT64_FORMAT " to %" G_GINT64_FORMAT,
                       long_str, G_MININT64, G_MAXINT64);
 
         g_free (long_str);
@@ -426,7 +426,7 @@ _pygi_marshal_from_py_int64 (PyGIInvokeState   *state,
     }
 
     if (long_ < G_MININT64 || long_ > G_MAXINT64) {
-        PyErr_Format (PyExc_ValueError, "%lld not in range %ld to %ld", long_, G_MININT64, G_MAXINT64);
+        PyErr_Format (PyExc_ValueError, "%" G_GINT64_FORMAT " not in range %" G_GINT64_FORMAT " to %" G_GINT64_FORMAT, long_, G_MININT64, G_MAXINT64);
         return FALSE;
     }
 
@@ -457,10 +457,10 @@ _pygi_marshal_from_py_uint64 (PyGIInvokeState   *state,
 
 #if PY_VERSION_HEX < 0x03000000
     if (PyInt_Check (py_long)) {
-        long long_ = PyInt_AsLong (py_long);
+        gint64 long_ = (gint64) PyInt_AsLong (py_long);
         if (long_ < 0) {
-            PyErr_Format (PyExc_ValueError, "%ld not in range %d to %lu",
-                          long_, 0, G_MAXUINT64);
+            PyErr_Format (PyExc_ValueError, "%" G_GUINT64_FORMAT " not in range %d to %" G_GUINT64_FORMAT,
+                          (gint64) long_, 0, G_MAXUINT64);
             return FALSE;
         }
         ulong_ = long_;
@@ -498,7 +498,7 @@ _pygi_marshal_from_py_uint64 (PyGIInvokeState   *state,
             Py_DECREF (py_str);
         }
 
-        PyErr_Format (PyExc_ValueError, "%s not in range %d to %lu",
+        PyErr_Format (PyExc_ValueError, "%s not in range %d to %" G_GUINT64_FORMAT,
                       long_str, 0, G_MAXUINT64);
 
         g_free (long_str);
@@ -506,7 +506,7 @@ _pygi_marshal_from_py_uint64 (PyGIInvokeState   *state,
     }
 
     if (ulong_ > G_MAXUINT64) {
-        PyErr_Format (PyExc_ValueError, "%lu not in range %d to %lu", ulong_, 0, G_MAXUINT64);
+        PyErr_Format (PyExc_ValueError, "%" G_GUINT64_FORMAT " not in range %d to %" G_GUINT64_FORMAT, ulong_, 0, G_MAXUINT64);
         return FALSE;
     }
 
@@ -628,8 +628,8 @@ _pygi_marshal_from_py_unichar (PyGIInvokeState   *state,
     }
 
     if (size != 1) {
-       PyErr_Format (PyExc_TypeError, "Must be a one character string, not %ld characters",
-                     size);
+       PyErr_Format (PyExc_TypeError, "Must be a one character string, not %" G_GINT64_FORMAT " characters",
+                     (gint64) size);
        g_free (string_);
        return FALSE;
     }
@@ -736,6 +736,33 @@ _pygi_marshal_from_py_filename (PyGIInvokeState   *state,
     return TRUE;
 }
 
+static gpointer
+_pygi_arg_to_hash_pointer (const GIArgument *arg,
+                           GITypeTag        type_tag)
+{
+    switch (type_tag) {
+        case GI_TYPE_TAG_INT8:
+            return GINT_TO_POINTER(arg->v_int8);
+        case GI_TYPE_TAG_UINT8:
+            return GINT_TO_POINTER(arg->v_uint8);
+        case GI_TYPE_TAG_INT16:
+            return GINT_TO_POINTER(arg->v_int16);
+        case GI_TYPE_TAG_UINT16:
+            return GINT_TO_POINTER(arg->v_uint16);
+        case GI_TYPE_TAG_INT32:
+            return GINT_TO_POINTER(arg->v_int32);
+        case GI_TYPE_TAG_UINT32:
+            return GINT_TO_POINTER(arg->v_uint32);
+        case GI_TYPE_TAG_UTF8:
+        case GI_TYPE_TAG_FILENAME:
+        case GI_TYPE_TAG_INTERFACE:
+            return arg->v_pointer;
+        default:
+            g_critical("Unsupported type %s", g_type_tag_to_string(type_tag));
+            return arg->v_pointer;
+    }
+}
+
 gboolean
 _pygi_marshal_from_py_array (PyGIInvokeState   *state,
                              PyGICallableCache *callable_cache,
@@ -985,7 +1012,7 @@ _pygi_marshal_from_py_glist (PyGIInvokeState   *state,
                                  &item))
             goto err;
 
-        list_ = g_list_prepend (list_, item.v_pointer);
+        list_ = g_list_prepend (list_, _pygi_arg_to_hash_pointer (&item, sequence_cache->item_cache->type_tag));
         continue;
 err:
         /* FIXME: clean up list
@@ -1052,7 +1079,7 @@ _pygi_marshal_from_py_gslist (PyGIInvokeState   *state,
                             &item))
             goto err;
 
-        list_ = g_slist_prepend (list_, item.v_pointer);
+        list_ = g_slist_prepend (list_, _pygi_arg_to_hash_pointer (&item, sequence_cache->item_cache->type_tag));
         continue;
 err:
         /* FIXME: Clean up list
@@ -1070,23 +1097,6 @@ err:
     return TRUE;
 }
 
-static gpointer
-_pygi_arg_to_hash_pointer (const GIArgument *arg,
-                           GITypeTag        type_tag)
-{
-    switch (type_tag) {
-        case GI_TYPE_TAG_INT32:
-            return GINT_TO_POINTER(arg->v_int32);
-        case GI_TYPE_TAG_UTF8:
-        case GI_TYPE_TAG_FILENAME:
-        case GI_TYPE_TAG_INTERFACE:
-            return arg->v_pointer;
-        default:
-            g_critical("Unsupported type %s", g_type_tag_to_string(type_tag));
-            return arg->v_pointer;
-    }
-}
-
 gboolean
 _pygi_marshal_from_py_ghash (PyGIInvokeState   *state,
                              PyGICallableCache *callable_cache,
@@ -1438,10 +1448,18 @@ _pygi_marshal_from_py_interface_struct (PyGIInvokeState   *state,
     } else if (!PyObject_IsInstance (py_arg, iface_cache->py_type)) {
         /* first check to see if this is a member of the expected union */
         if (!_is_union_member (iface_cache, py_arg)) {
-            if (!PyErr_Occurred())
-                PyErr_Format (PyExc_TypeError, "Expected %s, but got %s",
+            if (!PyErr_Occurred()) {
+                PyObject *module = PyObject_GetAttrString(py_arg, "__module__");
+
+                PyErr_Format (PyExc_TypeError, "argument %s: Expected %s, but got %s%s%s",
+                              arg_cache->arg_name ? arg_cache->arg_name : "self",
                               iface_cache->type_name,
-                              iface_cache->py_type->ob_type->tp_name);
+                              module ? PYGLIB_PyUnicode_AsString(module) : "",
+                              module ? "." : "",
+                              py_arg->ob_type->tp_name);
+                if (module)
+                    Py_DECREF (module);
+            }
 
             return FALSE;
         }
@@ -1490,9 +1508,16 @@ _pygi_marshal_from_py_interface_object (PyGIInvokeState   *state,
     }
 
     if (!PyObject_IsInstance (py_arg, ( (PyGIInterfaceCache *)arg_cache)->py_type)) {
-        PyErr_Format (PyExc_TypeError, "Expected %s, but got %s",
+        PyObject *module = PyObject_GetAttrString(py_arg, "__module__");
+
+        PyErr_Format (PyExc_TypeError, "argument %s: Expected %s, but got %s%s%s",
+                      arg_cache->arg_name ? arg_cache->arg_name : "self",
                       ( (PyGIInterfaceCache *)arg_cache)->type_name,
-                      ( (PyGIInterfaceCache *)arg_cache)->py_type->ob_type->tp_name);
+                      module ? PYGLIB_PyUnicode_AsString(module) : "",
+                      module ? "." : "",
+                      py_arg->ob_type->tp_name);
+        if (module)
+            Py_DECREF (module);
         return FALSE;
     }
 
@@ -1534,12 +1559,18 @@ gboolean _pygi_marshal_from_py_interface_instance (PyGIInvokeState   *state,
             if (!PyObject_IsInstance (py_arg, iface_cache->py_type)) {
                 /* wait, we might be a member of a union so manually check */
                 if (!_is_union_member (iface_cache, py_arg)) {
-                    if (!PyErr_Occurred())
+                    if (!PyErr_Occurred()) {
+                        PyObject *module = PyObject_GetAttrString(py_arg, "__module__");
                         PyErr_Format (PyExc_TypeError,
-                                      "Expected a %s, but got %s",
+                                      "argument %s: Expected a %s, but got %s%s%s",
+                                      arg_cache->arg_name ? arg_cache->arg_name : "self",
                                       iface_cache->type_name,
+                                      module ? PYGLIB_PyUnicode_AsString(module) : "",
+                                      module ? "." : "",
                                       py_arg->ob_type->tp_name);
-
+                        if (module)
+                            Py_DECREF (module);
+                    }
                     return FALSE;
                 }
             }
@@ -1565,9 +1596,15 @@ gboolean _pygi_marshal_from_py_interface_instance (PyGIInvokeState   *state,
                 GType expected_type = iface_cache->g_type;
 
                 if (!g_type_is_a (obj_type, expected_type)) {
-                    PyErr_Format (PyExc_TypeError, "Expected a %s, but got %s",
+                    PyObject *module = PyObject_GetAttrString(py_arg, "__module__");
+                    PyErr_Format (PyExc_TypeError, "argument %s: Expected %s, but got %s%s%s",
+                                  arg_cache->arg_name ? arg_cache->arg_name : "self",
                                   iface_cache->type_name,
+                                  module ? PYGLIB_PyUnicode_AsString(module) : "",
+                                  module ? "." : "",
                                   py_arg->ob_type->tp_name);
+                    if (module)
+                        Py_DECREF (module);
                     return FALSE;
                 }
             }
