@@ -12,7 +12,8 @@ import locale
 import subprocess
 from io import StringIO, BytesIO
 
-from gi.repository import GObject, GLib
+import gi
+from gi.repository import GObject, GLib, Gio
 
 from gi.repository import GIMarshallingTests
 
@@ -1873,6 +1874,69 @@ class TestInterfaces(unittest.TestCase):
                                  GIMarshallingTests.Interface2):
             pass
 
+    def test_type_mismatch(self):
+        obj = GIMarshallingTests.Object()
+
+        # wrong type for first argument: interface
+        enum = Gio.File.new_for_path('.').enumerate_children(
+            '', Gio.FileQueryInfoFlags.NONE, None)
+        try:
+            enum.next_file(obj)
+            self.fail('call with wrong type argument unexpectedly succeeded')
+        except TypeError as e:
+            # should have argument name
+            self.assertTrue('cancellable' in str(e), e)
+            # should have expected type
+            self.assertTrue('xpected Gio.Cancellable' in str(e), e)
+            # should have actual type
+            self.assertTrue('GIMarshallingTests.Object' in str(e), e)
+
+        # wrong type for self argument: interface
+        try:
+            Gio.FileEnumerator.next_file(obj, None)
+            self.fail('call with wrong type argument unexpectedly succeeded')
+        except TypeError as e:
+            if sys.version_info < (3, 0):
+                self.assertTrue('FileEnumerator' in str(e), e)
+                self.assertTrue('Object' in str(e), e)
+            else:
+                # should have argument name
+                self.assertTrue('self' in str(e), e)
+                # should have expected type
+                self.assertTrue('xpected Gio.FileEnumerator' in str(e), e)
+                # should have actual type
+                self.assertTrue('GIMarshallingTests.Object' in str(e), e)
+
+        # wrong type for first argument: GObject
+        var = GLib.Variant('s', 'mystring')
+        action = Gio.SimpleAction.new('foo', var.get_type())
+        try:
+            action.activate(obj)
+            self.fail('call with wrong type argument unexpectedly succeeded')
+        except TypeError as e:
+            # should have argument name
+            self.assertTrue('parameter' in str(e), e)
+            # should have expected type
+            self.assertTrue('xpected GLib.Variant' in str(e), e)
+            # should have actual type
+            self.assertTrue('GIMarshallingTests.Object' in str(e), e)
+
+        # wrong type for self argument: GObject
+        try:
+            Gio.SimpleAction.activate(obj, obj)
+            self.fail('call with wrong type argument unexpectedly succeeded')
+        except TypeError as e:
+            if sys.version_info < (3, 0):
+                self.assertTrue('SimpleAction' in str(e), e)
+                self.assertTrue('Object' in str(e), e)
+            else:
+                # should have argument name
+                self.assertTrue('self' in str(e), e)
+                # should have expected type
+                self.assertTrue('xpected Gio.Action' in str(e), e)
+                # should have actual type
+                self.assertTrue('GIMarshallingTests.Object' in str(e), e)
+
 
 class TestInterfaceClash(unittest.TestCase):
 
@@ -2292,3 +2356,18 @@ class TestModule(unittest.TestCase):
         self.assertTrue('SimpleStruct' in output, output)
         self.assertTrue('Interface2' in output, output)
         self.assertTrue('method_array_inout' in output, output)
+
+
+class TestProjectVersion(unittest.TestCase):
+    def test_version_str(self):
+        self.assertGreaterEqual(gi.__version__, "3.3.5")
+
+    def test_version_info(self):
+        self.assertEqual(len(gi.version_info), 3)
+        self.assertGreaterEqual(gi.version_info, (3, 3, 5))
+
+    def test_check_version(self):
+        self.assertRaises(ValueError, gi.check_version, (99, 0, 0))
+        self.assertRaises(ValueError, gi.check_version, "99.0.0")
+        gi.check_version((3, 3, 5))
+        gi.check_version("3.3.5")
