@@ -8,15 +8,32 @@ sys.path.insert(0, "../")
 
 from compathelper import _long, _unicode, _bytes
 
-from gi.repository import GLib
-from gi.repository import GObject
-from gi.repository import Gdk
-from gi.repository import Gtk
-from gi.repository import Gio
-from gi.repository import Pango
-from gi.repository import GdkPixbuf
 import gi.overrides as overrides
 import gi.types
+from gi.repository import GLib
+from gi.repository import GObject
+from gi.repository import Gio
+
+try:
+    from gi.repository import GdkPixbuf
+    GdkPixbuf  # pyflakes
+except ImportError:
+    GdkPixbuf = None
+
+try:
+    from gi.repository import Atk, Pango
+    (Atk, Pango)  # pyflakes
+except ImportError:
+    Pango = None
+
+Gtk = None
+if Pango:
+    try:
+        from gi.repository import Gdk
+        from gi.repository import Gtk
+        Gtk  # pyflakes
+    except ImportError:
+        Gtk = None
 
 
 class TestRegistry(unittest.TestCase):
@@ -470,6 +487,7 @@ class TestGLib(unittest.TestCase):
         self.assertEqual(repr(v), "GLib.Variant('(is)', (1, 'somestring'))")
 
 
+@unittest.skipUnless(Pango, 'Pango not available')
 class TestPango(unittest.TestCase):
 
     def test_default_font_description(self):
@@ -491,6 +509,7 @@ class TestPango(unittest.TestCase):
         self.assertEqual(layout.get_text(), "Foobar")
 
 
+@unittest.skipUnless(Gtk, 'Gtk not available')
 class TestGdk(unittest.TestCase):
 
     def test_constructor(self):
@@ -598,6 +617,7 @@ class TestGdk(unittest.TestCase):
         self.assertRaises(ValueError, Gdk.Cursor, 1, 2, 3)
 
 
+@unittest.skipUnless(Gtk, 'Gtk not available')
 class TestGtk(unittest.TestCase):
 
     def test_container(self):
@@ -1392,6 +1412,10 @@ class TestGtk(unittest.TestCase):
         self.assertFalse(p1 <= None)
 
         self.assertEqual(tuple(p1), (1, 2, 3))
+        self.assertEqual(p1[0], 1)
+        self.assertEqual(p1[1], 2)
+        self.assertEqual(p1[2], 3)
+        self.assertRaises(IndexError, p1.__getitem__, 3)
 
     def test_tree_model(self):
         tree_store = Gtk.TreeStore(int, str)
@@ -1566,6 +1590,15 @@ class TestGtk(unittest.TestCase):
         self.assertRaises(IndexError, tree_store.__delitem__, -101)
         self.assertRaises(IndexError, tree_store.__delitem__, 101)
 
+    def test_tree_model_get_iter_fail(self):
+        # TreeModel class with a failing get_iter()
+        class MyTreeModel(GObject.GObject, Gtk.TreeModel):
+            def do_get_iter(self, iter):
+                return (False, None)
+
+        tm = MyTreeModel()
+        self.assertEqual(tm.get_iter_first(), None)
+
     def test_tree_model_edit(self):
         model = Gtk.ListStore(int, str, float)
         model.append([1, "one", -0.1])
@@ -1648,6 +1681,32 @@ class TestGtk(unittest.TestCase):
                            text=0,
                            style=2)
 
+    def test_tree_view_column_set_attributes(self):
+        store = Gtk.ListStore(int, str)
+        directors = ['Fellini', 'Tarantino', 'Tarkovskiy']
+        for i, director in enumerate(directors):
+            store.append([i, director])
+
+        treeview = Gtk.TreeView()
+        treeview.set_model(store)
+
+        column = Gtk.TreeViewColumn()
+        treeview.append_column(column)
+
+        cell = Gtk.CellRendererText()
+        column.pack_start(cell, expand=True)
+        column.set_attributes(cell, text=1)
+
+        # might cause a Pango warning, do not break on this
+        old_mask = GLib.log_set_always_fatal(
+            GLib.LogLevelFlags.LEVEL_CRITICAL | GLib.LogLevelFlags.LEVEL_ERROR)
+        # This will make cell.props.text receive a value, otherwise it
+        # will be None.
+        treeview.get_preferred_size()
+        GLib.log_set_always_fatal(old_mask)
+
+        self.assertTrue(cell.props.text in directors)
+
     def test_tree_selection(self):
         store = Gtk.ListStore(int, str)
         for i in range(10):
@@ -1751,6 +1810,25 @@ class TestGtk(unittest.TestCase):
         self.assertFalse(start.ends_tag())
         self.assertFalse(start.toggles_tag())
 
+    def test_text_buffer_search(self):
+        buffer = Gtk.TextBuffer()
+        buffer.set_text('Hello World Hello GNOME')
+
+        i = buffer.get_iter_at_offset(0)
+        self.assertTrue(isinstance(i, Gtk.TextIter))
+
+        self.assertEqual(i.forward_search('world', 0, None), None)
+
+        (start, end) = i.forward_search('World', 0, None)
+        self.assertEqual(start.get_offset(), 6)
+        self.assertEqual(end.get_offset(), 11)
+
+        (start, end) = i.forward_search('world',
+                                        Gtk.TextSearchFlags.CASE_INSENSITIVE,
+                                        None)
+        self.assertEqual(start.get_offset(), 6)
+        self.assertEqual(end.get_offset(), 11)
+
     def test_buttons(self):
         self.assertEqual(Gtk.Button, overrides.Gtk.Button)
 
